#!/bin/sh
malia=arm64
malim=bifrost
malis=rockchip
maliv=r8p0

malix=bifrost_for_linux-${maliv}

malid=drivers/gpu/arm/bifrost_for_linux

. ./build-env.sh

export ARCH=arm64
if [ "X${BSP_ARCH}" = "Xarmhf" ]; then
  export ARCH=arm
fi

defcfg=${BSP_CONFIG}_defconfig

defcfg=arch/${ARCH}/configs/${defcfg}

KDIR=$(pwd)/linux


gitpatches() {
  if [ -e ../patches/$2 ]; then
    echo ""
    echo "Apply series patches/$2"
    echo ""

    mkdir -p patches/$1

    l=none
    for p in `cat ../patches/$2 | grep -v '^#' | grep patch` ; do
      b=`basename $p`
      d=`dirname $p`
      l=${p}
      n=`echo $b | sed s/\.patch$//g`

      if [ -e patches/${p} ]; then
        true
      elif [ -e ../patches/${p} ]; then
        cp -p ../patches/${p} patches/${p}
      else
        cp -p  ../patches/${b} patches/${p}
      fi

      echo "Apply patch patches/${p}"

      #if grep -q '\\r' patches/${p} ; then
      #  sed -i -e 's/\r$//g' patches/${p}
      #fi

      if echo $n | grep -q -E '^mali-' ; then
        if [ -e ${malid} ]; then
          sed -i "s|/src/devicedrv/|/${malid}/|g" patches/${p}
        fi
      fi

      if grep -q "/drivers/gpu/arm/midgard_for_linux/" patches/${p} ; then
        if [ -e ${malid} ]; then
          sed -i "s|/drivers/gpu/arm/midgard_for_linux/|/${malid}/|g" patches/${p}
          sed -i s/' midgard '/' bifrost '/g patches/${p}
          sed -i "s/ r1.p. / ${maliv} /g" patches/${p}
        fi
      fi

      nogitcp=1
      gitcpcont=0
      nogitam=1
      gitamcont=0
      if false ; then
      if grep -m1 -q "^From ${n} Mon Sep 17 00:00:00 2001" patches/${p} ; then
        rc=0
        git cherry-pick ${n} || rc=$?
        echo "git cherry-pick ${n}: rc=${rc}"
        if [ $rc = 0 ] ; then
          nogitcp=0
          echo ${p} >> patches/series-cherry-pick
        elif [ $rc != 128 ] ; then
          #gitcpcont=1
          git cherry-pick --abort
        fi
      fi
      fi

      if [ $nogitcp = 1 -a $gitcpcont = 0 ]; then
      if grep -m1 -q "^From ........................................ Mon Sep 17 00:00:00 2001" patches/${p} ; then
        c=`grep -m1 "^From ........................................ Mon Sep 17 00:00:00 2001" patches/${p} | sed s/'^From '//g | cut -d ' ' -f1`

        sed -i -e "0,/^---$/ s/^---$/\n${d} ${c}\n---/" patches/${p}

        if git am --keep-cr < patches/${p} ; then
          nogitam=0
          echo ${p} >> patches/series-am
        else
          gitamcont=1
          #git am --skip
        fi
      fi
      fi

      if [ $nogitam = 1 -a $nogitcp = 1 ]; then
        pb=`echo ${p} | sed s/'\.patch$'/'-bin.patch'/g`

        #echo "Check patches/${pb}"
        if [ -e ../patches/${pb} ]; then
          cp -p ../patches/${pb} patches/${pb}
          echo git apply --stat --apply patches/${pb}
          git apply --stat --apply patches/${pb}
        fi

        if ! git apply --stat --apply patches/${p} ; then
          #git apply --stat --apply --3way patches/${p}
          #sed -i -e 's/\r$//g' patches/${p}
          patch -p1 --no-backup-if-mismatch < patches/${p}
          echo ${p} >> patches/series-no-git
        else
          echo ${p} >> patches/series-no-am
        fi
      fi

      if echo $n | grep -q -E '^linux-' ; then
        n=`echo $n | sed s/^linux-4\.1.\.y-//g`
        n=`echo $n | sed s/^linux-4\.4\.y-//g`
        n=`echo $n | sed s/^linux-4\.1.-//g`

      elif echo $n | grep -q -E '^mali-' ; then
        n=`echo $n | sed s/^mali-r.p.-//g`

      elif grep -m1 -q "^From ${n} Mon Sep 17 00:00:00 2001" patches/${p} ; then
        if ! grep -q '^---$' patches/${p} ; then
          exit 1
        fi

      else
        exit 1
      fi

      if [ $nogitam = 1 -a $nogitcp = 1 ]; then
        echo git commit -m "${d} ${n}"
        git add .
        #git commit -m "${d} ${n}"
        #echo "${d} ${n}" > patches/${p}.msg

        if grep -m1 -q "^From ........................................ Mon Sep 17 00:00:00 2001" patches/${p} ; then
          if ! grep -q '^---$' patches/${p} ; then
            exit 1
          fi
          cat patches/${p} | grep -m1 -B399 '^---$' | sed /^---$/d | grep -m1 -A399 '^Subject:' | sed s/'^Subject: '/''/g > patches/${p}.msg
        else
          echo "${d} ${n}" > patches/${p}.msg
        fi
        
        if [ $gitcpcont = 1 ]; then
          git cherry-pick --continue
        elif [ $gitamcont = 1 ]; then
          git am --continue
        else
          git commit -F patches/${p}.msg
        fi
      fi

      echo ""
    done

    echo "Now in patch patches/${l}"
    cat ../patches/$2 >> patches/series

    #quilt push -a
  fi

}

applypatches() {
  gitpatches $1 $2
  #quiltpatches $1 $2
}


if [ -e ${KDIR}/drivers/gpu ]; then
  echo "Integrating ${malis} ${malim} driver..."

  cd ${KDIR}

  if [ ! -e ${malid} ]; then
    tar xzf ../mali-bifrost_for_linux-rockchip.tar.gz
    git add drivers/gpu/arm
    git commit -m "drm: add ${malim} ${malis} ${maliv} driver"

    applypatches ${malix} series-mali-${malix}
  fi

  if [ -e drivers/gpu/arm/bifrost_for_linux ]; then
    modgpuarm=false

    if ! grep -q 'MALI_BIFROST_FOR_LINUX' ${KDIR}/drivers/gpu/arm/Kconfig ; then
      cat <<EOFMALIDEFCFG | tee -a ${KDIR}/drivers/gpu/arm/Kconfig

config MALI_BIFROST_FOR_LINUX
	bool "Mali Bifrost for Linux only"

EOFMALIDEFCFG
      modgpuarm=true
    fi

    if ! grep -q 'arm/bifrost_for_linux/' ${KDIR}/drivers/gpu/arm/Kconfig ; then
      echo 'source "drivers/gpu/arm/bifrost_for_linux/Kconfig"' >> ${KDIR}/drivers/gpu/arm/Kconfig
      modgpuarm=true
    fi

    if ! grep -q 'bifrost_for_linux/' ${KDIR}/drivers/gpu/arm/Kbuild ; then
      echo 'obj-$(CONFIG_MALI_BIFROST_FOR_LINUX) += bifrost_for_linux/' >> ${KDIR}/drivers/gpu/arm/Kbuild
      modgpuarm=true
    fi

    if grep -q 'drivers/gpu/arm/bifrost/' drivers/gpu/arm/bifrost_for_linux/Kconfig ; then
      sed -i "s|drivers/gpu/arm/bifrost/|drivers/gpu/arm/bifrost_for_linux/|g" drivers/gpu/arm/bifrost_for_linux/Kconfig
      modgpuarm=true
    fi

    if $modgpuarm ; then
      #cd ${KDIR}
      git add drivers/gpu/arm
      git commit -m "drm: modify ${malim} ${malis} ${maliv} build"
      #cd - > /dev/null
    fi
  fi
fi

echo OK

